library IEEE;
use IEEE.STD_LOGIC_1164.ALL;



entity FactorielMem is
	Generic ( N : positive := 108; 
			  M : positive := 5;
			  P : positive := 8
			);
    Port ( Rst 			: in  STD_LOGIC;
           Clk 			: in  STD_LOGIC;
           D_in 		: in  STD_LOGIC_VECTOR (M-1 downto 0);
		   D_out_Ready 	: out  STD_LOGIC;
           D_out 		: out  STD_LOGIC_VECTOR (P-1 downto 0));
end FactorielMem;

architecture Behavioral of FactorielMem is

signal D_outReady_tmp 	:   STD_LOGIC :='0';
signal D_out_tmp 		:   STD_LOGIC_VECTOR (N+M-1 downto 0):=x"000000000000000000000000000" & b"00001";
signal D_in_tmp 		:   STD_LOGIC_VECTOR (M-1 downto 0):=b"00001";

COMPONENT FactorielN
PORT(
	Rst : IN std_logic;
	Clk : IN std_logic;
	D_in : IN std_logic_vector(M-1 downto 0);          
	D_out_Ready : OUT std_logic;
	D_out : OUT std_logic_vector(N+M-1 downto 0)
	);
END COMPONENT;

begin

	FAC5: FactorielN PORT MAP(
		Rst => Rst,
		Clk => Clk,
		D_in => D_in_tmp,
		D_out_Ready => D_outReady_tmp,
		D_out => D_out_tmp 
	);
	 
	-- Memorisation de la sortie + Gestion de 0!  
	P_data_out : process(Rst, Clk, D_outReady_tmp)
	begin
		if Rst ='1' then 
			D_out(P-1 downto 1) <=(others =>'0');
			D_out(0) <= '1'; 
		elsif Clk = '1' and Clk'event then 
			-- 0!=1
			if D_in=b"00000" then
				D_in_tmp <= b"00001";
			else	
				D_in_tmp<= D_in;
			end if;
			-- Saturation de la sortie 
			if D_outReady_tmp='1' then 
				if D_out_tmp > x"000000000000000000000000008" & b"00000" then  -- 256
					D_out <= x"FF";
				else
					D_out <= D_out_tmp(P-1 downto 0);
				end if;
			end if;
		end if;
	end process;
	D_out_Ready <=D_outReady_tmp;
end Behavioral;

